import 'package:bip32_ed25519/api.dart';
import 'package:fast_base58/fast_base58.dart';
import 'package:pointycastle/pointycastle.dart';

class CesiumWallet {
  final String salt;
  final String password;
  final Uint8List seed;
  final SigningKey rootKey;

  CesiumWallet._(
      {required this.salt,
      required this.password,
      required this.seed,
      required this.rootKey});

  factory CesiumWallet(String salt, String password) {
    var scrypt = KeyDerivator('scrypt');
    scrypt.init(
      ScryptParameters(
        4096,
        16,
        1,
        32,
        Uint8List.fromList(salt.codeUnits),
      ),
    );
    var seedFactory = scrypt.process(Uint8List.fromList(password.codeUnits));
    SigningKey rootKeyFactory = SigningKey(seed: seedFactory);
    // Bip32SigningKey rootKeyFactory =
    //     Bip32SigningKey.normalizeBytes(seedFactory);
    return CesiumWallet._(
        salt: salt,
        password: password,
        seed: seedFactory,
        rootKey: rootKeyFactory);
  }

  String getPubkey() {
    return Base58Encode(rootKey.publicKey);
  }

  SignatureBase sign(String document) {
    return rootKey.sign(document.codeUnits.toUint8List()).signature;
  }

  bool verifySign(String document, SignatureBase signature) {
    return rootKey.verifyKey
        .verify(signature: signature, message: document.codeUnits.toUint8List());
  }
}
